#
# Copyright (C) 1999-2000 VMware, Inc.
# All Rights Reserved
#
# $Id: Server.pm,v 1.23 2000/10/20 18:52:52 brettv Exp $

package VMware::Control::Server;

use strict;

require DynaLoader;

use vars qw($VERSION @ISA);

$VERSION = '0.02';

@ISA = qw(DynaLoader);

use VMware::Control '0.02'; #$VERSION; XXX do we need this?
#bootstrap VMware::Control $VERSION;

# Preloaded methods go here.

sub new {
    my ($hostname, $port, $username, $password) = (undef, 0, undef, undef);

    $hostname = shift if $#_ >= 0;
    $port = shift if $#_ >= 0;
    $username = shift if $#_ >= 0;
    $password = shift if $#_ >= 0;

    if (!defined($port)) {
        $port = 0;
    }
            
    return New($hostname, $port, $username, $password);
}

sub get_last_error {
    my $server = shift;
    my ($ret, $string) = $server->GetLastError();
    if (wantarray) {
        return ($ret, $string);
    } else {
        return $ret;
    }
}

sub connect {
    my $server = shift;
    my $ret = $server->Connect();
    if ($ret) {
        return $ret;
    } else {
        return undef;
    }
}

sub is_connected {
    my $server = shift;
    return $server->IsConnected();
}

sub enumerate {
    my $server = shift;
    my %hash;
    my @values = $server->Enumerate();

    # Hack to remove duplicates XXX
    my @unique_values;
    foreach my $value (@values) {
        if (!defined($hash{$value})) {
            $hash{$value} = 1;
            push @unique_values, $value;
        }
    }
    return @unique_values;
}

sub disconnect {
    my $server = shift;
    my $ret = $server->Disconnect();
    if ($ret) {
        return $ret;
    } else {
        return undef;
    }
}

sub register {
    my $server = shift;
    my $cfg = shift;
    my $ret = $server->Register($cfg);
    if ($ret) {
        return $ret;
    } else {
        return undef;
    }
}

sub exec {
    my $server = shift;
    my $xmlargs = shift;
    my $ret = $server->Exec($xmlargs);
    if ($ret) {
        return $ret;
    } else {
        return undef;
    }
}

#Initialize the API library and return 1 if OK
VMware::Control::Server::Init();
__END__

=head1 NAME

VMware::Control::Server - Programmatically manipulate a server running
VMware virtual machines.

=head1 SYNOPSIS

  use VMware::Control::Server;

  # Connect to a remote machine
  $server = VMware::Control::Server::new("vmware.companyX.com", 902, 
                                         $username, $password);
  $err = $server->connect();

  # Connect to the local machine as the current user
  $server = VMware::Control::Server::new();
  $err = $server->connect();
  if (!defined($err)) {
    ($errorNumber, $errorString) = $server->get_last_error();
  }

  @vm_identifiers = $server->enumerate();

  $server->disconnect();

=head1 DESCRIPTION

This package provides an interface for connecting to a VMware server
product and listing the virtual machines available on the server.

CAUTION: This API is preliminary and will change significantly between
this Beta and the final release version.

(This package in particular is far from complete -- future versions
will provide more access to information about the server and more
control over the server.)

=item VMware::Control::Server::new($hostname, $port, $username, $password)

Connects to the given hostname and network port and authenticates with
the supplied username and password.  If $hostname is not given or
undef, the authentication is performed for the local machine.  In this
case, the username and password are optional: if they are not supplied
the current user is authenticated to VMware.  (Otherwise, username and
password may still be supplied to authenticate as a different user.)

=item $server->connect()

Attempt to establish a connection to the server.

=item $server->get_last_error()

When a method fails (that is, returns undef), this method will return
the error code corresponding to the failure, along with a string
description (often with more specific error information).

=item $server->enumerate()

This method enumerates the VMs on the server, and returns a list of VM
identifiers.  Given a VM_IDENTIFIER, a VM object can be created using
VMware::Control::VM.  The results are not returned in any particular order.

C<$server-E<gt>enumerate> only returns identifiers for VMs for which you
have appropriate access rights. Thus it is possible that enumerate
will not return a complete list of the VMs on the current machine.
See L<ACCESS CONTROL> for more information.

Returns a list of VM_IDENTIFIERS.

=item $server->disconnect()

Close the connection to the server.

=head1 AUTHORS

John Haggerty, Daniel Kerns, Brett Vasconcellos.

=head1 COPYRIGHT

    (c) 2000 VMware Incorporated.  All rights reserved.

=head1 VERSION

Version 0.2	   21 Sep  2000

C<% pod2html --header --title="VMware::Control::Server" E<lt> Server.pm E<gt> perlAPI-server.html>

=head1 SEE ALSO

perl(1).

=cut
